/* ------------------------------------------------------------------------------------------------ */
/* FILE :		BareMetalA53.c																		*/
/*																									*/
/* CONTENTS :																						*/
/*				Example code for bare-metal applications using the evaluation version of			*/
/*				uAbassi.																			*/
/*				This file contains the uAbassi wrapper												*/
/*				Serial port settings:																*/
/*							Baud Rate: 115200														*/
/*							Data bits:      8														*/
/*							Stop bits:      1														*/
/*							Parity:      none														*/
/*							Flow Ctrl:   none														*/
/*							Emulation: VT-100														*/
/*																									*/
/*																									*/
/* Copyright (c) 2017-2019, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.9 $																				*/
/*	$Date: 2019/02/19 01:02:04 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* NOTE:																							*/
/*		This is the bare-metal application start-up code.											*/
/*		You should be able to re-use it as iS for your own purposes									*/
/*		To run on a different hardware than the one(s) provided in this package, refer to			*/
/*		the file CreatingNewTarget.txt.																*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#include "uAbassi.h"
#include "SysCall.h"								/* System Call layer stuff						*/

#include "Platform.h"								/* Everything about the target processor is here*/
#include "HWinfo.h"									/* Everything about the tagret hardware is here	*/


#if  ((((OS_PLATFORM) & 0x00FFFFFF) == 0x0000AA10)	/* Arria 10									*/	\
 ||   (((OS_PLATFORM) & 0x00FFFFFF) == 0x0000AAC5))	/* Arria 5 & Cyclone V						*/
  #include "dw_uart.h"
#elif ((((OS_PLATFORM) & 0x00FFFFFF) == 0x00007020)	/* Xilinx's Zynq							*/	\
   ||  (((OS_PLATFORM) & 0x00FFFFFF) == 0x00007753))/* Xilinx's UltraScale+ A53					*/
  #include "cd_uart.h"
#elif (((OS_PLATFORM) & 0x00FFFFFF) == 0x0000FEE6)	/* NXP (Freescale) i.MX6					*/
  #include "imx_uart.h"
#endif

/* ------------------------------------------------------------------------------------------------ */
/* App definitions																					*/

/* ------------------------------------------------------------------------------------------------ */
/* App variables																					*/

int G_UartDevIn  = UART_DEV;						/* Needed by the system call layer if used		*/
int G_UartDevOut = UART_DEV;						/* Needed by the system call layer if used		*/
int G_UartDevErr = UART_DEV;						/* Needed by the system call layer if used		*/

/* ------------------------------------------------------------------------------------------------ */
/* App functions																					*/

extern void App_0(void);							/* Code running on core #0 (BareMetalApps.c)	*/
extern void App_1(void);							/* Code running on core #1 (BareMetalApps.c)	*/
extern void App_2(void);							/* Code running on core #2 (BareMetalApps.c)	*/
extern void App_3(void);							/* Code running on core #3 (BareMetalApps.c)	*/

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

int main(void)
{
TSK_t *Task;										/* Task created for each of the non #0 cores	*/

													/* By default, libraries flush the I/O buffer	*/
	setvbuf(stdin,  NULL, _IONBF, 0);				/* when full or when new-line					*/
	setvbuf(stdout, NULL, _IONBF, 0);				/* If you want line buffering, change _IOBNF	*/
	setvbuf(stderr, NULL, _IONBF, 0);				/* to _IOLBF									*/

/* ------------------------------------------------ */
/* System set-up									*/

	OSstart();										/* Start uAbassi, the RTOS						*/

	TSKsetCore(TSKmyID(), 0);						/* Make sure App_0 will always run on Core #0	*/
													/* App_0 is called at the end on main()			*/
	SysCallInit();									/* Initialize the System Call layer				*/

	OSintOn();										/* Enable the interrupts						*/

/* ------------------------------------------------ */
/* SysTick Set-up									*/
													/* Timer tick needed for Sleep() & time-outs	*/
	OSisrInstall(OS_TICK_INT, &TIMtick);			/* Install the RTOS timer tick function			*/
	TIMERinit(((((OS_TICK_SRC_FREQ/OS_TICK_PRESCL)/100000)*(OX_TIMER_US))/(10)));
	GICenable(-OS_TICK_INT, 128, 0);				/* Prio=mid, level trig, all cores interrupted	*/

/* ------------------------------------------------ */
/* UART set-up										*/
/* 8 bits, No Parity, 1 Stop						*/

   #if (0U == ((UART_LIST_DEVICE) & (1U << (UART_DEV))))
	#error "Selected UART device is not in UART_LIST_DEVICE"
   #endif

	uart_init(UART_DEV, BAUDRATE, 8, 0, 10, (UART_Q_SIZE_RX), (UART_Q_SIZE_TX),UART_FILT_OUT_LF_CRLF);

   #if ((UART_DEV) == 0)							/* If the stdio UART_DEV is UART #0				*/
	OSisrInstall(UART0_INT, &UARTintHndl_0);		/* Install the UART interrupt handler			*/
	GICenable(-UART0_INT, 128, 0);					/* UART Must use level and not edge trigger		*/
   #endif											/* Int # -ve: the interrupt targets all cores	*/

   #if ((UART_DEV) == 1)							/* If the stdio UART_DEV is UART #1				*/
	OSisrInstall(UART1_INT, &UARTintHndl_1);		/* Install the UART interrupt handler			*/
	GICenable(-UART1_INT, 128, 0);					/* UART Must use level and not edge trigger		*/
   #endif											/* Int # -ve: the interrupt targets all cores	*/

   #if ((UART_DEV) == 2)							/* If the stdio UART_DEV is UART #2				*/
	OSisrInstall(UART2_INT, &UARTintHndl_2);		/* Install the UART interrupt handler			*/
	GICenable(-UART2_INT, 128, 0);					/* UART Must use level and not edge trigger		*/
   #endif											/* Int # -ve: the interrupt targets all cores	*/

   #if ((UART_DEV) == 3)							/* If the stdio UART_DEV is UART #3				*/
	OSisrInstall(UART3_INT, &UARTintHndl_3);		/* Install the UART interrupt handler			*/
	GICenable(-UART3_INT, 128, 0);					/* UART Must use level and not edge trigger		*/
   #endif											/* Int # -ve: the interrupt targets all cores	*/

/* ------------------------------------------------ */
/* Application set-up								*/

	Task = TSKcreate("App Core 1", 0, 16384, &App_1, 0);/* Create the task for core #1				*/
	if (Task == NULL) {
		printf("Something is wrong (1)\n");
		for(;;);
	}
	TSKsetCore(Task, 1);							/* Force it to always run on core #1			*/
	TSKresume(Task);								/* when BMP (G_OS_MP_TYPE == 4 or 5)			*/

  #if ((OS_N_CORE) > 2)
	Task = TSKcreate("App Core 2", 0, 16384, &App_2, 0);/* Create the task for core #1				*/
	if (Task == NULL) {
		printf("Something is wrong (2)\n");
		for(;;);
	}
	TSKsetCore(Task, 2);							/* Force it to always run on core #2			*/
	TSKresume(Task);								/* when BMP (G_OS_MP_TYPE == 4 or 5)			*/
  #endif

  #if ((OS_N_CORE) > 3)
	Task = TSKcreate("App Core 3", 0, 16384, &App_3, 0);/* Create the task for core #1				*/
	if (Task == NULL) {
		printf("Something is wrong (3)\n");
		for(;;);
	}
	TSKsetCore(Task, 3);							/* Force it always run on core #3				*/
	TSKresume(Task);								/* when BMP (G_OS_MP_TYPE == 4 or 5)			*/
  #endif

	Task = TSKcreate("Should Fail", 0, 16384, &App_0, 0);
	if (Task != NULL) {								/* You will not see this message because		*/
		MTXlock(MTXopen("Printf Mtx"), -1);			/* uAbassi for baremetal supports exactly the	*/
		printf("Wrong uAbassi selected\n");			/* same number of tasks as the number of cores	*/
		for(;;);									/* MTXlock() is used to busy out printf so the 	*/
	}												/* other cores won't be able to print messages	*/

	App_0();										/* Go into core #0 bare-metal code				*/

	for(;;);										/* Should never come back here					*/
}

/* ------------------------------------------------------------------------------------------------ */
/* Not used in this demo but needed to build uAbassi												*/

void TIMcallBack(void)
{
	return;
}

/* EOF */
